;*************************************************************************

;                      Timer I Watchdog Timer Usage

;  This program demonstrates how to use Timer I on the 83C751 or 83C752 
;  microcontrollers as a watchdog timer when the I2C port is not used. 
;  Once started, Timer I must be cleared more often than once every 1024
;  machine cycles. If Timer I is allowed to overflow, a Timer I 
;  interrupt will be generated. Thus, if global interrupts or the Timer 
;  I interrupt are inhibited, the watchdog function will be disabled. 
;  Also, if the watchdog interrupt occurs during another interrupt 
;  service, it will be delayed until an RETI (return from interrupt) 
;  instruction is executed. The I2C bus pins SCL and SDA may be used as 
;  open drain outputs.

;*************************************************************************

$MOD751
$Title(Timer I Watchdog)
$Date(08-29-89)
$Debug

          ORG     0
          AJMP    Reset

          ORG     1Bh                 ;Timer I interrupt. 
TimerI:   CLR     EA                  ;Get here only if watchdog overflows.
          CLR     TIRUN               ;Turn off Timer I.
          SETB    CLRTI               ;Clear Timer I interrupt.
          DEC     SP                  ;Remove return address from stack.
          DEC     SP
          ACALL   XRETI               ;Force interrupt pending to clear.
          AJMP    Reset               ;Do a warm start.
XRETI:    RETI

Reset:    MOV     I2CFG,#0            ;Clear I2CFG.
          SETB    TIRUN               ;Enable Timer I run.
          SETB    ETI                 ;Enable Timer I interrupt.
          SETB    EA                  ;Enable interrupt system.

Loop:     ACALL   Delay1              ;Wait 901 machine cycles.
          ACALL   WdRst               ;Reset Watchdog.
          ACALL   Delay1              ;Wait 901 machine cycles.
          ACALL   WdRst               ;Reset Watchdog.
          ACALL   Delay1              ;Wait 901 + 4 for ACALL & prior RET.
          ACALL   Delay2              ;Wait 108 + 2 for ACALL.
          NOP     ;1016 cycles.
          NOP     ;1017 cycles.
          NOP     ;1018 cycles.
          NOP     ;1019 cycles.
          NOP     ;1020 cycles.
          NOP     ;1021 cycles.
          NOP     ;1022 cycles.
          NOP     ;1023 cycles.
          NOP     ;1024 cycles : Should get 'bitten' by watchdog here.
          NOP     ;1025 cycles.
          NOP     ;1026 cycles.
          NOP     ;1027 cycles.
          NOP     ;1028 cycles.
          NOP     ;1029 cycles.
          AJMP    Loop                ;Should never get here.

WdRst:    CLR     TIRUN               ;Reset Watchdog timer (Timer I).
          SETB    TIRUN
          RET

Delay1:   MOV     A,#128              ;Wait 901 machine cycles (1).
          SJMP    DLoop               ;(2)
Delay2:   MOV     A,#15               ;Wait 108 machine cycles (1).
DLoop:    INC     DPTR                ;Delay = (ACC * 7) + 2 mach. cyc (2).
          INC     DPTR                ;(2)
          DEC     A                   ;(1)
          JNZ     Dloop               ;(2)
          RET                         ;(2)

          END
